/******************************************************************************
 * ESO/DMD
 *
 * "@(#) $Id: ngamsArchiveClient.c,v 1.27 2006/02/01 16:59:38 jknudstr Exp $" 
 *
 * who       when      what
 * --------  --------  --------------------------------------------------
 * jknudstr  22/12/03  created
 */

/************************************************************************
 *   NAME
 *   ngamsArchiveClient - NG/AMS Archive Client
 *
 *   SYNOPSIS
 *   Invoke the tool without command parameters to get a help page.
 *
 *   DESCRIPTION
 *   Small client application used to archive files into the NGAS System,
 *   possibly located on a geographically different site than the data 
 *   provider.
 *
 *   FILES
 *
 *   ENVIRONMENT
 *
 *   COMMANDS
 *
 *   RETURN VALUES
 *
 *   CAUTIONS 
 *
 *   EXAMPLES
 *
 *   SEE ALSO
 *
 *   BUGS   
 * 
 *-----------------------------------------------------------------------
 */

static char *rcsId="@(#) $Id: ngamsArchiveClient.c,v 1.27 2006/02/01 16:59:38 jknudstr Exp $"; 
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include <unistd.h>
#include <strings.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>
#include <sys/time.h>
#include <unistd.h>

#include "ngams.h"
#include "ngamsArchiveClient.h"


char *ngamsArchCliManPage(void);


void ngamsCorrectUsage()
{
    printf("%s", (ngamsArchCliManPage() + 1));   
}


ngamsSTAT ngamsExecCmd(const char     *cmd,
		       ngamsHUGE_BUF  res)
{
    FILE          *fp;

    memset(res, 0, sizeof(ngamsHUGE_BUF));
    ngamsLogInfo(LEV4, "Executing command: %s ...", cmd);
    if ((fp = popen(cmd, "r")) == NULL)
	{
	ngamsLogError("Error executing command: %s", cmd);
	return ngamsSTAT_FAILURE;
	}
    fread(res, sizeof(ngamsHUGE_BUF), sizeof(char), fp);
    ngamsLogInfo(LEV4, "Result of cmd: %s: %s", cmd, res);
    pclose(fp);

    return ngamsSTAT_SUCCESS;
}


char *ngamsSetGetRootDir(const char  *dir)
{
    static ngamsMED_BUF  _rootDir;
    if (dir != NULL)
	{
	strcpy(_rootDir, dir);
	if (_rootDir[strlen(_rootDir) - 1] == '/')
	    _rootDir[strlen(_rootDir) - 1] = '\0';
	}
    return _rootDir;
}


char *ngamsGetQueueDir(void)
{
    static ngamsMED_BUF  _queueDir;
    if (_queueDir[0] == '\0')
	sprintf(_queueDir, "%s/%s/%s", ngamsSetGetRootDir(NULL),
		ngamsARCH_CLI_DIR, ngamsARCH_CLI_QUE_DIR);
    return _queueDir;
}


char *ngamsGetArchDir(void)
{
    static ngamsMED_BUF  _archivedDir;
    if (_archivedDir[0] == '\0')
	sprintf(_archivedDir, "%s/%s/%s", ngamsSetGetRootDir(NULL),
		ngamsARCH_CLI_DIR, ngamsARCH_CLI_ARC_DIR);
    return _archivedDir;
}


char *ngamsGetBadDir(void)
{
    static ngamsMED_BUF  _badDir;
    if (_badDir[0] == '\0')
	sprintf(_badDir, "%s/%s/%s", ngamsSetGetRootDir(NULL),
		ngamsARCH_CLI_DIR, ngamsARCH_CLI_BAD_DIR);
    return _badDir;
}


char *ngamsGetLogDir(void)
{
    static ngamsMED_BUF  _logDir;
    if (_logDir[0] == '\0')
	sprintf(_logDir, "%s/%s/%s", ngamsSetGetRootDir(NULL),
		ngamsARCH_CLI_DIR, ngamsARCH_CLI_LOG_DIR);
    return _logDir;
}


char *getPidFile(void)
{    
    static int          initialized = 0;
    static ngamsMED_BUF tmpName;  
    
    if (!initialized)
	{
	sprintf(tmpName, "%s/%s/.ngamsArchiveClient-PID",
		ngamsSetGetRootDir(NULL), ngamsARCH_CLI_DIR);
	initialized = 1;
	}
    return tmpName;
}


void ngamsArchCliSignalHandler(int sigNo)
{
    ngamsLogInfo(LEV1, "Received signal: %d", sigNo);
    remove(getPidFile());
    ngamsLogInfo(LEV1, "Terminating ...");  
    exit(0);
}


void ngamsAlarmHandler(int sigNo)
{
    ngamsLogInfo(LEV4, "Woken up - performing server loop ...");
}


ngamsSTAT ngamsArchCliInit(const char            *rootDir,
			   const ngamsLOG_LEVEL  logLevel,
			   const int             logRotate,
			   const int             logHistory,
			   const ngamsLOG_LEVEL  verboseLevel)
{ 
    char           *dirs[] = {"",
			      ngamsARCH_CLI_LOG_DIR,
			      ngamsARCH_CLI_QUE_DIR,
			      ngamsARCH_CLI_ARC_DIR,
			      ngamsARCH_CLI_BAD_DIR,
                              NULL};
    int            i = 0, tmpFd;
    ngamsMED_BUF   tmpName, rootDirLoc, tmpHostId, systemId, pidBuf;
    ngamsSTAT      stat;

    /* Create the directory structure */
    strcpy(rootDirLoc, ngamsSetGetRootDir(rootDir));
    while (dirs[i] != NULL)
	{
	sprintf(tmpName, "%s/%s/%s", rootDirLoc, ngamsARCH_CLI_DIR, dirs[i]);
	/* Check if directory already exists */
	if ((tmpFd = open(tmpName, O_RDONLY, 0)) == -1)
	    {
	    if (mkdir(tmpName, ngamsSTD_DIR_PERMS) == -1)
		{
		fprintf(stderr, "\nCould not create directory: %s", tmpName);
		return ngamsSTAT_FAILURE;
		}
	    }
	else
	    close(tmpFd);
	i++;
	}

    /* Store PID of the Archive Client in file in its working directory */
    remove(getPidFile());
    if ((tmpFd = creat(getPidFile(), 0666)) == -1)
	{
	fprintf(stderr, "\nCould not create PID file: %s", getPidFile());
	return ngamsSTAT_FAILURE;
	}
    memset(pidBuf, 0, sizeof(ngamsMED_BUF));
    sprintf(pidBuf, "%d", (int)getpid());
    write(tmpFd, pidBuf, strlen(pidBuf));
    close(tmpFd);

    /* Set up logging properties */
    ngamsSetVerboseLevel(verboseLevel);
    sprintf(tmpName, "%s/%s/%s/%s", rootDirLoc, ngamsARCH_CLI_DIR, 
	    ngamsARCH_CLI_LOG_DIR, ngamsARCH_CLI_LOG_FILE);
    if ((stat = ngamsPrepLog(tmpName, logLevel, 
			     logRotate, logHistory)) != ngamsSTAT_SUCCESS)
	return stat;
    ngamsGetHostName(tmpHostId);
    sprintf(systemId, "%s@%s", ngamsARCH_CLI_NAME, tmpHostId);
    ngamsLogInfo(LEV1, "Initializing NG/AMS Archive Client - SYSTEM-ID: %s ",
		 systemId);

    /* Set up signal exit handlers */
    signal(SIGINT, ngamsArchCliSignalHandler);
    signal(SIGTERM, ngamsArchCliSignalHandler);
    signal(SIGHUP, ngamsArchCliSignalHandler);

    ngamsLogInfo(LEV1, "NG/AMS Archive Client initialized");
    return ngamsSTAT_SUCCESS;
}


ngamsSTAT ngamsMoveFile2StatDir(const char  *statDir,
				const char  *filename,
				const char  *queueFilename,
				const char  *xmlStat)
{
    ngamsMED_BUF   trgFilename, xmlStatDocFile, isoTime;

    ngamsGenIsoTime(3, isoTime);
    sprintf(trgFilename, "%s/%s___%s", statDir, isoTime, filename);
    sprintf(xmlStatDocFile, "%s/%s___%s%s", statDir, isoTime, 
	    filename, ngamsARCH_CLI_STAT_EXT);
    ngamsLogInfo(LEV1, "Moving handled file (or link): %s to "
		 "directory: %s ...", queueFilename, trgFilename);
    if (rename(queueFilename, trgFilename) == -1)
	{
	ngamsLogError("Serious error ocurred moving handled " 
		      "file: %s from Archive Queue Directory to " 
		      "directory: %s! Terminating!", 
		      queueFilename, trgFilename);
	return ngamsSTAT_FAILURE;
	}
    ngamsLogInfo(LEV1, "Moved handled file (or link): %s to the "
		 "directory: %s ...", queueFilename, trgFilename);

    ngamsLogInfo(LEV1, "Creating XML document with status from NG/AMS: %s",
		 xmlStatDocFile);
    if (ngamsSaveInFile(xmlStatDocFile, xmlStat) == ngamsSTAT_FAILURE)
	return ngamsSTAT_FAILURE;

    return ngamsSTAT_SUCCESS;
}    


void ngamsGenNgamsStatSum(const ngamsSTATUS  *status,
			  ngamsHUGE_BUF      statBuf)
{
    sprintf(statBuf, "NG/AMS Status: Date %s - Error Code: %d - Host ID: %s "
	    "- Status: %s - State: %s - Sub-State: %s - Version: %s - "
	    "Message: %s", status->date, status->errorCode, status->hostId,
	    status->status, status->state, status->subState, status->version,
	    status->message);
}


ngamsSTAT ngamsCheckArchiveQueue(char                  *remoteHost,
				 const int             remotePort,
				 const char            *checksum,
				 ngamsPAR_ARRAY        *parArray)
{
    DIR              *dirPtr = NULL;
    struct dirent    *dirEnt = NULL;
    ngamsHUGE_BUF    ngamsStatBuf, checksumRes;
    ngamsMED_BUF     queueFilename, checksumCmd; 
    ngamsSTAT        stat;
    ngamsSTATUS      status;

    if ((dirPtr = opendir(ngamsGetQueueDir())) == NULL)
	{    
	ngamsLogError("Error opening Archive Queue Directory: %s",
		      ngamsGetQueueDir());
	goto errExit;
	}
    while ((dirEnt = readdir(dirPtr)) != NULL)
	{
	if (dirEnt->d_name[0] != '.')
	    {
	    sprintf(queueFilename, "%s/%s", ngamsGetQueueDir(),dirEnt->d_name);
	    ngamsLogInfo(LEV1, "Attempting to archive file: %s ...", 
			 queueFilename);
	    if (*checksum != '\0')
		{
		sprintf(checksumCmd, "%s %s", checksum, queueFilename);
		if (ngamsExecCmd(checksumCmd,checksumRes) != ngamsSTAT_SUCCESS)
		    {
		    ngamsLogError("Error generating checksum using Checksum " 
				  "Plug-In: %s", checksum);
		    continue;
		    }
		ngamsAddParAndVal(parArray, "checksum_util", 
				  ngamsEncodeUrlVal(checksum, 1));
		ngamsAddParAndVal(parArray, "checksum_result", 
				  ngamsEncodeUrlVal(checksumRes, 1));
		ngamsLogInfo(3,"Checksum of file: %s generated with "
			     "Checksum Plug-In: %s: %s", queueFilename, 
			     checksum, checksumRes);
		}
	    /*ngamsAddParAndVal(&parArray, "no_versioning", "1");*/
	    stat = ngamsGenSendData(remoteHost, remotePort, ngamsCMD_ARCHIVE,
				    ngamsNO_TIME_OUT, queueFilename, "", 
				    parArray, &status);
	    if ((stat == ngamsSTAT_FAILURE) || 
		(status.errorCode != ngamsSTAT_SUCCESS) ||
		(strstr(status.status, NGAMS_FAILURE) != NULL))
		{
		/* The following actions are taken:
		 *
		 * o Bad File:               Move file to Bad Files Directory.
		 * o File Back-Log Buffered: Move file to Archived Directory.
		 * o Other Errors:           Keep file in Archive Queue.
		 */
		if ((strstr(status.message,"NGAMS_ER_DAPI_BAD_FILE")!=NULL) ||
		    (strstr(status.message, "NGAMS_ER_UNKNOWN_MIME_TYPE1") !=
		     NULL))
		    {
		    ngamsLogError("File: %s was classified as bad by " 
				  "NG/AMS. Moving to Bad Files Directory.",
				  queueFilename);
		    ngamsGenNgamsStatSum(&status, ngamsStatBuf);
		    ngamsLogError(ngamsStatBuf);
		    if (ngamsMoveFile2StatDir(ngamsGetBadDir(),
					      dirEnt->d_name, 
					      queueFilename, 
					      status.replyData[0])
			== ngamsSTAT_FAILURE)
			goto errExit;
		    }
		else if (strstr(status.message, "NGAMS_WA_BUF_DATA") != NULL)
		    {
		    ngamsLogWarning("File: %s could not be archived. "
				    "File has been back-log buffered.",
				    queueFilename);
		    ngamsGenNgamsStatSum(&status, ngamsStatBuf);
		    ngamsLogInfo(LEV1, ngamsStatBuf);
		    if (ngamsMoveFile2StatDir(ngamsGetArchDir(),
					      dirEnt->d_name, 
					      queueFilename,
					      status.replyData[0])
			== ngamsSTAT_FAILURE)
			goto errExit;
		    }
		else
		    {
		    ngamsLogWarning("File: %s could not be archived. " 
				    "Leaving file in Archive Files Queue.",
				    queueFilename);
		    ngamsGenNgamsStatSum(&status, ngamsStatBuf);
		    ngamsLogWarning(ngamsStatBuf);
		    }
		}
	    else
		{
		/* Move the file to the Archived Files Directory 
		 */
		ngamsGenNgamsStatSum(&status, ngamsStatBuf);
		ngamsLogInfo(LEV1, ngamsStatBuf);
		if (ngamsMoveFile2StatDir(ngamsGetArchDir(), dirEnt->d_name, 
					  queueFilename, status.replyData[0]) 
		    == ngamsSTAT_FAILURE)
		    goto errExit;
		}
	    }
	}
    closedir(dirPtr);
    
    return ngamsSTAT_SUCCESS;

 errExit:
    if (dirPtr != NULL) free(dirPtr);
    if (dirEnt != NULL) free(dirEnt);
    return ngamsSTAT_FAILURE;
}


ngamsSTAT ngamsCleanUpArchivedFiles(char        *remoteHost,
				    const int   remotePort,
				    const int   cleanUpTimeOut)
{  
    DIR              *dirPtr = NULL;
    struct dirent    *dirEnt = NULL;
    struct stat      statBuf;
    struct timeval   timeNow;
    ngamsMED_BUF     archFile, xmlStatDocFile;
    ngamsHUGE_BUF    xmlStatBuf, msgBuf;
    ngamsPAR_ARRAY   parArray;
    ngamsSMALL_BUF   fileId, fileVer;
    ngamsSTATUS      status;
    
    if ((dirPtr = opendir(ngamsGetArchDir())) == NULL)
	{    
	ngamsLogError("Error opening Archived Files Directory: %s",
		      ngamsGetArchDir());
	goto errExit;
	}
    while ((dirEnt = readdir(dirPtr)) != NULL)
	{
	/* Handle only the actual files, the XML status documents are handled
	 * together with the files.
	 */
	if ((dirEnt->d_name[0] != '.') && 
	    (strstr(dirEnt->d_name, ngamsARCH_CLI_STAT_EXT) == NULL))
	    {
	    sprintf(archFile, "%s/%s", ngamsGetArchDir(), dirEnt->d_name);
	    sprintf(xmlStatDocFile, "%s%s", archFile, ngamsARCH_CLI_STAT_EXT);

	    /* Check if the time for keeping the file in the Archived Files
	     * Directory has elapsed
	     */
	    if (stat(xmlStatDocFile, &statBuf) == -1)
		{
		ngamsLogError("Error querying file status for file: %s",
			      archFile);
		goto errExit;
		}
	    gettimeofday(&timeNow, NULL);
	    if ((timeNow.tv_sec - statBuf.st_ctime) < cleanUpTimeOut)
		continue;
	    ngamsLogInfo(LEV1, "File in Archived Files Directory: "
			 "%s could be removed (time from creation expired) "
			 "...", archFile);
	    
	    /* Get File ID + File Version from the XML status document
	     */ 
	    if (ngamsLoadFile(xmlStatDocFile, xmlStatBuf, 
			      sizeof(ngamsHUGE_BUF)) == ngamsSTAT_FAILURE)
		{    
		ngamsLogError("Error loading XML status document: %s",
			      xmlStatDocFile);
		goto errExit;
		}
	    if (ngamsGetXmlAttr(xmlStatBuf, "FileStatus", "FileId",
				sizeof(ngamsSMALL_BUF), 
				fileId) == ngamsSTAT_FAILURE)
		{    
		ngamsLogError("Error error retrieving attribute "
			      "FileStatus:FileId from XML status document: %s",
			      xmlStatDocFile);
		goto errExit;
		}
	    if (ngamsGetXmlAttr(xmlStatBuf, "FileStatus", "FileVersion",
				sizeof(ngamsSMALL_BUF), 
				fileVer) == ngamsSTAT_FAILURE)
		{    
		ngamsLogError("Error error retrieving attribute "
			      "FileStatus:FileVersion from XML status "
			      "document: %s", xmlStatDocFile);
		goto errExit;
		}
	    if ((*fileId == '\0') || (*fileVer == '\0'))
		{
		if (ngamsGetXmlAttr(xmlStatBuf, "Status", "Message",
				    sizeof(ngamsHUGE_BUF), 
				    msgBuf) == ngamsSTAT_FAILURE)
		    {
		    ngamsLogError("Error error retrieving attribute "
				  "Status:Message from XML status "
				  "document: %s", xmlStatDocFile);
		    goto errExit;
		    }
		if (strstr(msgBuf, "NGAMS_WA_BUF_DATA") != NULL)
		    {
		    ngamsLogError("XML Status Document related to file: %s, "
				  "indicates that files was Back-Log " 
				  "Buffered. Remove status files manually.", 
				  archFile);
		    continue;
		    }
		else
		    {
		    ngamsLogError("Format of XML Status Document related " 
				  "to file: %s seems to be mal-formed. "
				  "Buffered. Remove status files manually.", 
				  archFile);
		    continue;
		    }
		}
 
	    /* Before deleting a file, it is checked if it is avaliable
	     * in the remote NGAS Archive
	     */ 
	    ngamsResetParArray(&parArray);
	    ngamsAddParAndVal(&parArray, "file_id", fileId);
	    ngamsAddParAndVal(&parArray, "file_version", fileVer);
	    if (ngamsGenSendCmd(remoteHost, remotePort, ngamsNO_TIME_OUT,
				ngamsCMD_CHECKFILE, &parArray,
				&status) == ngamsSTAT_FAILURE)
		{    
		ngamsLogError("Error sending command to NG/AMS Server: "
			      "%s/%d", remoteHost, remotePort);
		continue;
		}
	    /* IMPL: NGAMS_ER_FILE_OK kept for compatibity with V2.2 .
	     *       Can be removed from V2.3. */
	    if ((strstr(status.message, "NGAMS_INFO_FILE_OK") != NULL) ||
		(strstr(status.message, "NGAMS_ER_FILE_OK") != NULL))
		{
		/* The specified file is available in the NGAS Archive,
		 * we can go ahead and delete the file from the
		 * Archived Files Directory
		 */
		ngamsLogInfo(LEV1, "File in Archived Files Directory: %s, " 
			     "has been archived with File ID: %s and "
			     "File Version: %s. Removing copy.", archFile,
			     fileId, fileVer);
		remove(archFile);
		ngamsLogInfo(LEV1, "Removing File Archive Status XML " 
			     "Document file: %s", xmlStatDocFile);
		remove(xmlStatDocFile);
		}
	    else
		{
		ngamsLogInfo(LEV3, "File in Archived Files Directory: %s, " 
			     "which should have been archived with File ID: "
			     "%s and File Version: %s, is not in NGAS " 
			     "Archive. Keeping file in Archived Files Queue.",
			     archFile, fileId, fileVer);
		}
	    }
	}
    closedir(dirPtr);

    return ngamsSTAT_SUCCESS;

 errExit:
    if (dirPtr != NULL) closedir(dirPtr);
    if (dirEnt != NULL) free(dirEnt);
    return ngamsSTAT_FAILURE;
}


ngamsSTAT ngamsArchCliServerLoop(char                  *remoteHost,
				 const int             remotePort,
				 const char            *rootDir,
				 const int             pollTime,
				 const char            *checksum,
				 ngamsPAR_ARRAY        *parArray,
				 const int             cleanUpTimeOut,
				 const int             streams,
				 const int             archiveLog)
{
    ngamsMED_BUF  rotatedLogFile, archQueueLogFile;
    ngamsMED_BUF  dummyPath, logFilename, tmpHostId, systemId;

    ngamsGetHostName(tmpHostId);
    sprintf(systemId, "%s@%s", ngamsARCH_CLI_NAME, tmpHostId);
    ngamsLogInfo(LEV1, "Serving ...");
    while (1)
	{
	ngamsLogInfo(LEV4, "Checking Archive Queue: %s ...", 
		     ngamsGetQueueDir());
	if (ngamsCheckArchiveQueue(remoteHost, remotePort, 
				   checksum, parArray) == ngamsSTAT_FAILURE)
	    {
	    ngamsLogError("Serious error ocurred in server loop while "
			  "checking Archive File Queue! Terminating!");
	    return ngamsSTAT_FAILURE;
	    }

	ngamsLogInfo(LEV4, "Checking if Archive Files Directory should be "
		     "cleaned up ...");
	if (ngamsCleanUpArchivedFiles(remoteHost, 
				      remotePort,
				      cleanUpTimeOut) == ngamsSTAT_FAILURE)
	    {
	    ngamsLogError("Serious error ocurred in server loop while "
			  "cleaning up Archived Files Directory! " 
			  "Terminating!");
	    return ngamsSTAT_FAILURE;
	    }

	ngamsLogInfo(LEV4, "Checking if log file should be rotated ...");
	if (ngamsLogFileRotate(LEV1, systemId, 
			       rotatedLogFile) == ngamsSTAT_FAILURE)
	    ngamsLogError("Error ocurred rotating log file. Continuing "
			  "operation, but intervention is needed!");
	else if (archiveLog && (*rotatedLogFile != '\0'))
	    {
	    /* The log file should be archived. Simply copy in the rotated
	     * log file into the Archive Queue Directory, it will then
	     * be archived during the next loop.
	     */
	    ngamsSplitFilename(rotatedLogFile, dummyPath, logFilename);
	    sprintf(archQueueLogFile, "%s/%s", ngamsGetQueueDir(),logFilename);
	    if (symlink(rotatedLogFile, archQueueLogFile) == -1)
		ngamsLogError("Error ocurred creating link from rotated log " 
			      "file to the Archive Queue Directory %s->%s -- "
			      "log file not archived!",
			      rotatedLogFile, archQueueLogFile);
	    }

	ngamsLogInfo(LEV4, "Checking if log files should be removed ...");
	if (ngamsCleanUpRotLogFiles(LEV1) == ngamsSTAT_FAILURE)
	    ngamsLogError("Error ocurred cleaning up rotatedg log files. " 
			  "Continuing operation, but intervention is needed!");

	ngamsLogInfo(LEV4, "Suspending for %d seconds ...", pollTime);
	signal(SIGALRM, ngamsAlarmHandler);
	alarm(pollTime);
	pause();
	}
    ngamsLogInfo(LEV1, "Terminating ...");
    return ngamsSTAT_SUCCESS;
}


/* Main
 */
int main (int argc, char *argv[])
{
    int              i;
    int              port, pollTime, cleanUpTimeOut, streams;
    int              logLevel, logRotate, logHistory, archiveLog;
    int              verboseLevel;
    ngamsMED_BUF     tmpPar, tmpVal;
    ngamsMED_BUF     host, servers, rootDir, statStr, checksum, auth;
    ngamsPAR_ARRAY   parArray;
    ngamsSTAT        stat;


    /* Parse input parameters */
    *host = '\0';
    port = -1;
    *servers = '\0';
    *rootDir = '\0';
    *auth = '\0';
    pollTime = 30; 
    *checksum = '\0';
    cleanUpTimeOut = 604800;  /* = 7 days */
    streams = 1;
    logLevel = 3;
    logRotate = 43200;        /* = 12:00am */
    logHistory = 30;          /* = 30 days */
    archiveLog = 0;           /* Don't archive log files after rotation */
    verboseLevel = 0;
    ngamsResetParArray(&parArray);       
    for (i = 1; i < argc; i++)
	{
	strcpy(tmpPar, argv[i]);
	ngamsToUpper(tmpPar);
	if (strcmp(tmpPar, "-ARCHIVELOG") == 0)
	    archiveLog = 1;
	else if (strcmp(tmpPar, "-ARCHIVEPAR") == 0)
	    {
	    if (++i == argc) goto correctUsage;
	    if (*argv[i] == '-') goto correctUsage;
	    if (ngamsSplitParVal(argv[i], tmpPar, tmpVal) == ngamsSTAT_FAILURE)
		goto correctUsage;
	    ngamsAddParAndVal(&parArray, tmpPar, tmpVal);
	    } 
	else if (strcmp(tmpPar, "-AUTH") == 0)
	    {
	    if (++i == argc) goto correctUsage;
	    if (*argv[i] == '-') goto correctUsage;
	    strcpy(auth, argv[i]);
	    } 
	else if (strcmp(tmpPar, "-CHECKSUM") == 0)
	    {
	    if (++i == argc) goto correctUsage;
	    if (*argv[i] == '-') goto correctUsage;
	    strcpy(checksum, argv[i]);
	    } 
	else if (strcmp(tmpPar, "-CLEANUPTIMEOUT") == 0)
	    {
	    if (++i == argc) goto correctUsage;
	    if (*argv[i] == '-') goto correctUsage;
	    cleanUpTimeOut = atoi(argv[i]);
	    } 
	else if ((strcmp(tmpPar, "-H") == 0) || (strcmp(tmpPar, "-HELP") == 0))
	    goto correctUsage;
	else if (strcmp(tmpPar, "-HOST") == 0)
	    {
	    if (++i == argc) goto correctUsage;
	    if (*argv[i] == '-') goto correctUsage;
	    strcpy(host, argv[i]);
	    }
	else if (strcmp(tmpPar, "-PORT") == 0)
	    {
	    if (++i == argc) goto correctUsage;
	    if (*argv[i] == '-') goto correctUsage;
	    port = atoi(argv[i]);
	    }
	else if (strcmp(tmpPar, "-LICENSE") == 0)
	    {
	    printf("%s", ngamsLicense());
	    exit(0);
	    }
	else if (strcmp(tmpPar, "-LOGHISTORY") == 0)
	    {
	    if (++i == argc) goto correctUsage;
	    if (*argv[i] == '-') goto correctUsage;
	    logHistory = atoi(argv[i]);
	    }
	else if (strcmp(tmpPar, "-LOGLEVEL") == 0)
	    {
	    if (++i == argc) goto correctUsage;
	    if (*argv[i] == '-') goto correctUsage;
	    logLevel = atoi(argv[i]);
	    }
	else if (strcmp(tmpPar, "-LOGROTATE") == 0)
	    {
	    if (++i == argc) goto correctUsage;
	    if (*argv[i] == '-') goto correctUsage;
	    logRotate = atoi(argv[i]);
	    }
	else if (strcmp(tmpPar, "-POLLTIME") == 0)
	    {
	    if (++i == argc) goto correctUsage;
	    if (*argv[i] == '-') goto correctUsage;
	    pollTime = atoi(argv[i]);
	    if (pollTime < 1) pollTime = 1;
	    }
	else if (strcmp(tmpPar, "-ROOTDIR") == 0)
	    {
	    if (++i == argc) goto correctUsage;
	    if (*argv[i] == '-') goto correctUsage;
	    strcpy(rootDir, argv[i]);
	    }
	else if (strcmp(tmpPar, "-SERVERS") == 0)
	    {
	    if (++i == argc) goto correctUsage;
	    if (*argv[i] == '-') goto correctUsage;
	    strcpy(servers, argv[i]);
	    }
	else if (strcmp(tmpPar, "-STREAMS") == 0)
	    {
	    if (++i == argc) goto correctUsage;
	    if (*argv[i] == '-') goto correctUsage;
	    printf("\nNOTE: Command line option -streams is not yet "
		   "implemented!\n\n");
	    exit(1);
	    }
	else if (strcmp(tmpPar, "-V") == 0)
	    {
	    if (++i == argc) goto correctUsage;
	    if (*argv[i] == '-') goto correctUsage;
	    verboseLevel = atoi(argv[i]);
	    }
	else if (strcmp(tmpPar, "-VERSION") == 0)
	    { 
	    printf("%s\n", ngamsVersion());
	    exit(0);
	    }
	else
	    {
	    printf("\n\nFound illegal command line parameter: %s\n", tmpPar);
	    goto correctUsage;
	    }
	}   /* end-for (int i = 1; i < argc; i++) */
    if ((((*host == '\0') || (port == -1)) && (*servers == '\0')) ||
	(*rootDir == '\0'))
	goto correctUsage;
    if (*servers != '\0') 
	{
	if (ngamsParseSrvList(servers) != ngamsSTAT_SUCCESS) goto correctUsage;
	}

    /* Set Authorization key if specified */
    if (*auth != '\0') ngamsSetAuthorization(auth);

    /* Initialize + start to serve */
    if ((stat = ngamsArchCliInit(rootDir, logLevel, logRotate, logHistory,
				 verboseLevel)) != ngamsSTAT_SUCCESS)
	goto errExit;
    if ((stat = ngamsArchCliServerLoop(host, port, rootDir,
				       pollTime, checksum, &parArray,
				       cleanUpTimeOut, streams,
				       archiveLog)) != ngamsSTAT_SUCCESS)
	goto errExit;

    exit(0);
    return(0);


 correctUsage:
    ngamsCorrectUsage();
    exit(1);
    return(1);

 errExit:
    ngamsStat2Str(stat, statStr);
    if (*statStr)
	printf("\nError ocurred initializing NG/AMS Archive Client: "
	       "\n\n%s\n\n", statStr);
    else
	printf("\n\nError ocurred initializing NG/AMS Archive Client!\n\n");
    exit(1);
}


/* EOF */
