#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: ngamsEvent.py,v 1.3 2004/08/04 16:45:30 ngasmgr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  26/05/2003  Created
#

"""
Contains class to do event synchronization between threads.
"""

import threading


class ngamsEvent:
    """
    Class to do event synchronization between threads.
    """
    
    def __init__(self):
        """
        Constructor method.
        """
        self.__event = threading.Event()
        self.__eventInfoList = []


    def addEventInfo(self,
                     eventInfo):
        """
        Set the event info to be transferred from one thread to the other.

        eventInfo:    Event info (free format).

        Returns:      Reference to object itself (ngamsEvent).
        """
        self.__eventInfoList.append(eventInfo)
        return self


    def getEventInfoList(self):
        """
        Get the event info to be transferred from one thread to the other.
        
        Returns:      Event info (free format).
        """
        return self.__eventInfoList


    def isSet(self):
        """
        Returns 1  if the internal event flag is set.

        Returns:  Value of event flag (integer/0|1).
        """
        return self.__event.isSet()


    def set(self):
        """
        Set the internal event flag.

        Returns:   Reference to object itself (ngamsEvent).
        """
        self.__event.set()
        return self


    def clear(self):
        """
        Clear the internal event flag.

        Returns:   Reference to object itself (ngamsEvent).
        """
        self.__event.clear()
        self.__eventInfoList = []
        return self


    def wait(self,
             timeOut = None):
        """
        Wait for event flag to be set, possibly applying a timeout.

        timeOut:    Timeout in seconds (float).

        Returns:   Reference to object itself (ngamsEvent).
        """
        if (timeOut):
            self.__event.wait(timeOut)
        else:
            self.__event.wait()
        return self


# EOF
