#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: ngamsTestSuspensionPlugIn.py,v 1.9 2004/08/04 16:45:30 ngasmgr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  23/01/2002  Created
#

"""
Test Suspension Plug-In to simulate the NGAS host suspension.

This is done in the following way:
  - The host is marked as suspended in the DB (done by the Janitor Thread).
  - The suspension test plug-in enters a loop, checking if the host is
    marked as suspended.
  - When marked as not being suspended (by ngamsTestWakeUpPlugIn), the
    plug-in exists.

See also ngamsTestWakeUpPlugIn.py.
"""

# IMPL: Move this module to ngams/ngamsTest!

import time

from   ngams import *
import ngamsDiskInfo


def ngamsTestSuspensionPlugIn(srvObj):
    """
    Dummy Suspension Plug-In to test the handling of the NGAS host
    suspension.

    srvObj:         Reference to instance of the NG/AMS Server (ngamsServer).


    Returns:        Void.
    """
    info(4,"Test Suspension Plug-In executing ...")
    info(3,"Polling DB until server marked as not being suspended ...")
    startTime = time.time()
    logFlush()
    while (srvObj.getDb().getSrvSuspended(getHostId())): time.sleep(0.250)
    info(3,"NGAS Node: %s woken up after %.3fs of suspension" %\
         (getHostId(), (time.time() - startTime)))
    info(4,"Leaving Test Suspension Plug-In")
    logFlush()


# EOF
