#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: ngamsArchiveCmd.py,v 1.22 2005/08/31 08:48:57 jknudstr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  13/05/2003  Created
#

"""
Function to handle the ARCHIVE command.
"""


from ngams import *
import ngamsStatus, ngamsHighLevelLib, ngamsNotification, ngamsDiskUtils
import ngamsArchiveUtils
 

def handleCmdArchive(srvObj,
                     reqPropsObj,
                     httpRef):
    """
    Handle an ARCHIVE command.

    srvObj:         Reference to NG/AMS server class object (ngamsServer).
    
    reqPropsObj:    Request Property object to keep track of actions done
                    during the request handling (ngamsReqProps).
        
    httpRef:        Reference to the HTTP request handler
                    object (ngamsHttpRequestHandler).
        
    Returns:        Void.
    """
    info(4,"Entering handleCmdArchive() ...")

    # Is this NG/AMS permitted to handle Archive Requests?
    if (not srvObj.getCfg().getAllowArchiveReq()):
        errMsg = genLog("NGAMS_ER_ILL_REQ", ["Archive"])
        raise Exception, errMsg
    srvObj.checkSetState("Archive Request", [NGAMS_ONLINE_STATE],
                         [NGAMS_IDLE_SUBSTATE, NGAMS_BUSY_SUBSTATE],
                         NGAMS_ONLINE_STATE, NGAMS_BUSY_SUBSTATE)

    # Ensure we have the mime-type.
    if (reqPropsObj.getMimeType() == ""):
        mimeType = ngamsHighLevelLib.\
                   determineMimeType(srvObj.getCfg(), reqPropsObj.getFileUri())
        reqPropsObj.setMimeType(mimeType)
    else:
        mimeType = reqPropsObj.getMimeType()

    # This is a request probing for capability of handling the request.
    if (reqPropsObj.hasHttpPar("probe")):
        if (int(reqPropsObj.getHttpPar("probe"))):
            try:
                ngamsDiskUtils.findTargetDisk(srvObj.getDb(), srvObj.getCfg(),
                                              mimeType, sendNotification=0)
                msg = genLog("NGAMS_INFO_ARCH_REQ_OK",
                             [mimeType, getHostName()])
            except Exception, e:
                msg = genLog("NGAMS_ER_ARCH_REQ_NOK",
                             [mimeType, getHostName()])
            srvObj.reply(reqPropsObj, httpRef, NGAMS_HTTP_SUCCESS,
                         NGAMS_SUCCESS, msg)
            return

    # Act possibly as proxy for the Achive Request?
    try:
        archUnits = srvObj.getCfg().getStreamFromMimeType(mimeType).\
                    getHostIdList()
    except Exception, e:
        archUnits = []
    if (len(archUnits) > 0):
        targNodeName, targNode, targPort, targDiskObj =\
                      ngamsArchiveUtils.\
                      findTargetNode(srvObj.getDb(), srvObj.getCfg(),
                                     reqPropsObj.getMimeType())
        if ((targNodeName != getHostName()) or
            (int(targPort) != int(srvObj.getCfg().getPortNo()))):
            # Act as proxy, forward the request to the specified node.
            # IMPL: Support maybe HTTP redirection also for Archive Requests.
            httpStatCode, httpStatMsg, httpHdrs, data =\
                          srvObj.forwardRequest(reqPropsObj, httpRef,
                                                targNodeName, targPort, 1,
                                                mimeType)
            return   # Request handled at remote host and reply sent to client.

    # Handle Archive Request Locally.
    if (reqPropsObj.getHttpMethod() == NGAMS_HTTP_GET):
        info(1,"Handling Archive Pull Request ...")
        try:
            if (reqPropsObj.getFileUri() == ""):
                raise Exception, "No File URI/Filename specified!"
            reqPropsObj.setReadFd(ngamsArchiveUtils.\
                                  openCheckUri(reqPropsObj.getFileUri()))
            diskInfoObj = ngamsArchiveUtils.dataHandler(srvObj, reqPropsObj,
                                                        httpRef)
            srvObj.setSubState(NGAMS_IDLE_SUBSTATE)
            msg = "Successfully handled Archive Pull Request for data file " +\
                  "with URI: " + reqPropsObj.getSafeFileUri()
            info(1,msg)
            # If it is specified not to reply immediately (= to wait), we
            # send back a reply now.
            if (reqPropsObj.getWait()):
                srvObj.ingestReply(reqPropsObj, httpRef, NGAMS_HTTP_SUCCESS,
                                   NGAMS_SUCCESS, msg, diskInfoObj)
        except Exception, e:
            errMsg = genLog("NGAMS_ER_ARCHIVE_PULL_REQ",
                            [reqPropsObj.getSafeFileUri(), str(e)])
            error(errMsg)
            ngamsNotification.notify(srvObj.getCfg(), NGAMS_NOTIF_ERROR,
                                     "PROBLEM HANDLING ARCHIVE PULL REQUEST",
                                     errMsg)
            raise Exception, errMsg
    else:
        info(1,"Handling Archive Push Request ...")
        try:
            diskinfoObj = ngamsArchiveUtils.dataHandler(srvObj, reqPropsObj,
                                                        httpRef)
            msg = "Successfully handled Archive Push Request for " +\
                  "data file with URI: " + reqPropsObj.getSafeFileUri()
            info(4,msg)
            srvObj.setSubState(NGAMS_IDLE_SUBSTATE)
            
            # If it was specified not to reply immediately (= to wait),
            # we send back a reply now.
            if (reqPropsObj.getWait()):
                srvObj.ingestReply(reqPropsObj, httpRef,NGAMS_HTTP_SUCCESS,
                                   NGAMS_SUCCESS, msg, diskinfoObj)
        except Exception, e:
            errMsg = genLog("NGAMS_ER_ARCHIVE_PUSH_REQ",
                            [reqPropsObj.getSafeFileUri(), str(e)])
            error(errMsg)
            ngamsNotification.notify(srvObj.getCfg(), NGAMS_NOTIF_ERROR,
                                     "PROBLEM ARCHIVE HANDLING", errMsg)
            raise Exception, errMsg

    # Trigger Subscription Thread.
    srvObj.triggerSubscriptionThread()
        
    info(4,"Leaving handleCmdArchive()")


# EOF
