#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: ngamsExitCmdTest.py,v 1.15 2005/01/25 10:10:36 ngasmgr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  20/11/2002  Created
#

"""
This module contains the Test Suite for the EXIT Command.
"""

import os, sys
from   ngams import *
from   ngamsTestLib import *


class ngamsExitCmdTest(ngamsTestSuite):
    """
    Synopsis:
    Test Suite for the  EXIT Command.

    Description:
    The purpose of this Test Suite is to exercise the EXIT Command.
    Both the normal case where exit is granted should be tested, as well
    as when exit is not allowed (if the server is in Online State).

    Missing Test Cases:
    ...
    """

    def test_ExitCmd_1(self):
        """
        Synopsis:
        Normal execution EXIT Command/Offline.
        
        Description:
        Test that the server terminates if the EXIT Command is submitted
        while the server is in Offline State.

        Expected Result:
        The server in Offline State, should accept the EXIT Command and
        perform a clean shut down.

        Test Steps:
        - Start server (Auto Online = 0).
        - Submit EXIT Command.
        - Check that the response is as expected.
        - Check that the server is no longer running.
        
        Remarks:
        IMPL!: Test that the server is no longer running.
        """
        self.prepExtSrv(8888, 1, 1, 0)
        tmpStatFile = sendExtCmd(getHostName(), 8888, NGAMS_EXIT_CMD)
        refStatFile = "ref/ngamsCmdHandlingTest_test_handleCmdExit_1_ref"
        self.checkFilesEq(refStatFile, tmpStatFile, 
                          "Incorrect status returned for EXIT command")
        info(1,"IMPL: Check that NG/AMS Server has terminated")


    def test_ExitCmd_2(self):
        """
        Synopsis:
        Server in Online State -> EXIT Command rejected.
        
        Description:
        Test that the EXIT Command is rejected when submitted when the
        server is in Online State.

        Expected Result:
        The server should generate an Error Response indicating that the
        command cannot be handled when the server is in Online State.

        Test Steps:
        - Start server (Auto Online = 1).
        - Issue EXIT Command.
        - Check that the request is rejected.
        - Check that the server is still running.

        Remarks:
        IMPL!: Check that the server is still running after the EXIT Command.
        """
        self.prepExtSrv(8888, 1, 1, 1)
        tmpStatFile = sendExtCmd(getHostName(), 8888, NGAMS_EXIT_CMD)
        refStatFile = "ref/ngamsCmdHandlingTest_test_handleCmdExit_2_ref"
        self.checkFilesEq(refStatFile, tmpStatFile, 
                          "Incorrect status returned for EXIT command")
        info(1,"IMPL: Check that NG/AMS Server is still running")


def run():
    """
    Run the complete test.

    Returns:   Void.
    """
    runTest(["ngamsExitCmdTest"])


if __name__ == '__main__':
    """
    Main program executing the test cases of the module test.
    """
    runTest(sys.argv)


# EOF
