#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: ngamsSrvTestDynReqCallBack.py,v 1.2 2005/10/19 11:09:31 jknudstr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  07/10/2004  Created
#

"""
Child class of ngamsServer where the test case code can define how requests
are handled.
"""

import os, sys, time
from   ngams import *
import ngamsServer
from   ngamsTestLib import *


class ngamsServerTestDynReqCallBack(ngamsServer.ngamsServer):
    """
    Child class of ngamsServer where the test case code can define how requests
    are handled.
    """

    def __init__(self):
        """
        Constructor method.
        """
        ngamsServer.ngamsServer.__init__(self)


    def reqCallBack(self,
                    httpRef,
                    clientAddress,
                    method,
                    path,
                    requestVersion,
                    headers,
                    writeFd,
                    readFd):
        """
        Override ngamsServer.reqCallBack(). Simply load the name of the
        request handler to execute in the file written by the test case.
        """
        info(1,"Entering reqCallBack() ...")
        reqHandleCode = loadFile("tmp/reqCallBack_tmp")
        eval("self." + reqHandleCode +\
             "(httpRef, clientAddress, method, path, "+\
             "requestVersion, headers, writeFd, readFd)")

        
    def reqCallBack_BlockCmds1(self,
                               httpRef,
                               clientAddress,
                               method,
                               path,
                               requestVersion,
                               headers,
                               writeFd,
                               readFd):
        """
        Allow to execute EXIT, OFFLINE, STATUS. Block other commands.
        """
        info(1,"Entering reqCallBack_BlockCmds1() ...")
        info(1,"Handling command: %s ..." % path.strip().split("?")[0])
        if ((path.strip().find(NGAMS_EXIT_CMD) == 0) or
            (path.strip().find(NGAMS_OFFLINE_CMD) == 0) or
            (path.strip().find(NGAMS_STATUS_CMD) == 0)):
            ngamsServer.ngamsServer.reqCallBack(self, httpRef, clientAddress,
                                                method, path, requestVersion,
                                                headers, writeFd, readFd)
        else:
            info(1,"Blocking request ...")
            time.sleep(10e6)


    def reqCallBack_AccArchiveBlock2(self,
                                     httpRef,
                                     clientAddress,
                                     method,
                                     path,
                                     requestVersion,
                                     headers,
                                     writeFd,
                                     readFd):
        """
        Allow to execute ARCHIVE Commands (+ EXIT, OFFLINE, STATUS).
        Block RETRIEVE Commands + other commands.
        """
        info(1,"Entering reqCallBack_AccArchiveBlockRetrieve() ...")
        if ((path.strip().find(NGAMS_ARCHIVE_CMD) == 0) or
            (path.strip().find(NGAMS_EXIT_CMD) == 0) or
            (path.strip().find(NGAMS_OFFLINE_CMD) == 0) or
            (path.strip().find(NGAMS_STATUS_CMD) == 0)):
            info(1,"Handling command: %s ..." % path.strip().split("?")[0])
            ngamsServer.ngamsServer.reqCallBack(self, httpRef, clientAddress,
                                                method, path, requestVersion,
                                                headers, writeFd, readFd)
        else:
            info(1,"Blocking request ...")
            time.sleep(10e6)


    def reqCallBack_IllegalResp(self,
                                httpRef,
                                clientAddress,
                                method,
                                path,
                                requestVersion,
                                headers,
                                writeFd,
                                readFd):
        """
        Override ngamsServer.reqCallBack(). This version simply writes an
        illegal HTTP response.

        Only commands that are handled are: EXIT, OFFLINE, STATUS.
        """
        info(1,"Entering reqCallBack_IllegalResp() ...")
        if ((path.strip().find(NGAMS_EXIT_CMD) == 0) or
            (path.strip().find(NGAMS_OFFLINE_CMD) == 0) or
            (path.strip().find(NGAMS_STATUS_CMD) == 0)):
            info(1,"Handling command: %s ..." % path.strip().split("?")[0])
            ngamsServer.ngamsServer.reqCallBack(self, httpRef, clientAddress,
                                                method, path, requestVersion,
                                                headers, writeFd, readFd)
        else:
            info(1,"Sending back illegal HTTP response ...")
            time.sleep(0.500)
            readFd.flush()
            resp = loadFile("tmp/ngamsServerTestIllegalResp_tmp")
            writeFd.write(resp)

 
    def reqCallBack_SrvCrash1(self,
                              httpRef,
                              clientAddress,
                              method,
                              path,
                              requestVersion,
                              headers,
                              writeFd,
                              readFd):
        """
        Override ngamsServer.reqCallBack(). This version simply kills itself.
        The input socket it first flushed.

        Only command that is handled is: STATUS.
        """
        info(1,"Entering reqCallBack_SrvCrash1() ...")
        if (path.strip().find(NGAMS_STATUS_CMD) == 0):
            info(1,"Handling command: %s ..." % path.strip().split("?")[0])
            ngamsServer.ngamsServer.reqCallBack(self, httpRef, clientAddress,
                                                method, path, requestVersion,
                                                headers, writeFd, readFd)
        else:
            info(1,"Server killing itself ...")
            time.sleep(0.250)
            readFd.flush()
            self.killServer()
            sys.exit(0)


    def reqCallBack_SrvCrash2(self,
                              httpRef,
                              clientAddress,
                              method,
                              path,
                              requestVersion,
                              headers,
                              writeFd,
                              readFd):
        """
        Override ngamsServer.reqCallBack(). This version simply kills itself.
        The input socket it first flushed.

        Only command that is handled is: STATUS.
        """
        info(1,"Entering reqCallBack_SrvCrash1() ...")
        if ((path.strip().find(NGAMS_ARCHIVE_CMD) == 0) or
            (path.strip().find(NGAMS_STATUS_CMD) == 0)):
            info(1,"Handling command: %s ..." % path.strip().split("?")[0])
            ngamsServer.ngamsServer.reqCallBack(self, httpRef, clientAddress,
                                                method, path, requestVersion,
                                                headers, writeFd, readFd)
        else:
            info(1,"Server killing itself ...")
            time.sleep(0.250)
            readFd.flush()
            self.killServer()
            sys.exit(0)


 #    def reqCallBack_BrokSockRetrieve(self,
#                                      httpRef,
#                                      clientAddress,
#                                      method,
#                                      path,
#                                      requestVersion,
#                                      headers,
#                                      writeFd,
#                                      readFd):
#         """
#         Override ngamsServer.reqCallBack(). Simulate a broken socket connection
#         while sending back data to the client.

#         Accepted command(s): STATUS/ARCHIVE.
#         """
#         info(1,"Entering reqCallBack_SrvCrash1() ...")
#         if (path.strip().find(NGAMS_RETRIEVE_CMD) == -1):
#             info(1,"Handling command: %s ..." % path.strip().split("?")[0])
#             ngamsServer.ngamsServer.reqCallBack(self, httpRef, clientAddress,
#                                                 method, path, requestVersion,
#                                                 headers, writeFd, readFd)
#         else:
#             reqPropsObj = ngamsReqProps.ngamsReqProps()
#             self.addRequest(reqPropsObj)
#             reqPropsObj.setReadFd(readFd).setWriteFd(writeFd)
            

            

 
if __name__ == '__main__':
    """
    Main program executing the special test NG/AMS Server
    """
    ngamsTestSrv = ngamsServerTestDynReqCallBack()
    ngamsTestSrv.init(sys.argv)

# EOF
