#!/usr/bin/python
#******************************************************************************
# ESO/DMD
#
# "@(#) $Id: vultur_cascade_stat.py,v 1.2 2006/03/13 16:05:34 jknudstr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  01/04/2005  Created
#

"""
The Vultur Cascade Status Tool is used to query the status of a DAG on demand.
A reference to the DAG ID must be given.

SYNOPSIS:
The command line options for the tool are:

  $ vultur_cascade_stat --dagId=<ID>

The options are:

-dagId:        Overall ID for the DAG.


If the DAG is not running (completed, aborted or failed), the tool unblocks
immediately and indicates the current status.
"""

import sys, os, time, commands, glob
from vultur_utils import *


def correctUsage():
    """
    """
    print "\nCorrect usage is:\n\n" +\
          "$ vultur_cascade_stat --dagId=<ID>\n\n" +\
          "MANUAL PAGE:" + __doc__ + "\n"

def parsePars(argv):
    """
    """
    dagId = None
    idx = 1
    while idx < len(sys.argv):
        par = sys.argv[idx].upper()
        if (par.find("--DAGID") == 0):
            dagId = sys.argv[idx].split("=")[1].strip()
        elif ((par.find("-H") == 0) or (par.find("-HELP") == 0)):
            correctUsage()
            sys.exit(0)
        else:
            correctUsage()
            sys.exit(1)
        idx += 1

    if ((not dagId)):
        correctUsage()
        sys.exit(1)

    return (dagId)
    

def genDagStat(dagId):
    """
    """
    # Find the latest DOT file in connection with the job if available.
    bqsRoot = os.environ["BQSROOT"]
    user = os.environ["USER"]
    jobDirPat = os.path.normpath("%s/condor/%s-*" % (bqsRoot, dagId))
    dirList = glob.glob(jobDirPat)
    dirList.sort()
    if (dirList == []):
        return "No info found for job with Job ID: %s" % dagId
    jobDir = dirList[-1]
    dotFilePat = os.path.normpath("%s/%s.dot.*" % (jobDir, dagId))
    stat, out = commands.getstatusoutput("ls -ltr %s" % dotFilePat)
    if ((stat != 0) or (out.strip() == "")):
        return "No cascade status info available (yes)"
    lastDotFile = out.split("\n")[-1].split(" ")[-1]

    # Parse the chosen DOT file and generate a list of the jobs.
    report = "VULTUR CASCADE REPORT: %s\n\n" +\
             "Job Sub ID:                                                  " +\
             "Status:\n" +\
             80 * "-" + "\n"
    report = report % (dagId)
    dotLines = open(lastDotFile).readlines()
    for dotLine in dotLines:
        dotLine = dotLine.strip()
        if ((dotLine != "") and (dotLine.find("digraph") == -1) and
            (dotLine.find('label="DAGMan') == -1) and (dotLine != "}") and
            (dotLine.find(" -> ") == -1)):
            subJobId, val = dotLine.split(" [")
            status = val.split(" (")[-1].split(")")[0]
            if (status == "I"):
                stat = "Blocked"
            elif (status == "R"):
                stat = "Scheduled"
            elif (status == "Done"):
                stat = "Completed OK"
            else:
                stat = "Completed Failure"
            report += "%-60s %s\n" % (subJobId, stat)
    report += 80 * "-"
            
    return report


if __name__ == '__main__':
    """
    Main function.
    """
    dagId = parsePars(sys.argv)
    print genDagStat(dagId)
    sys.exit(0)

# EOF
