#!/usr/bin/python
#******************************************************************************
# ESO/DMD
#
# "@(#) $Id: vultur_submit_job.py,v 1.1.1.1 2005/12/08 16:38:31 jknudstr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  01/04/2005  Created
#

"""
The Vultur Job Submission Tool is used to submit a single job to Condor.

SYNOPSIS:
The command line options for the tool are:

  $ vultur_submit_job -jobId <ID> -executable <Executable> 
      [-args <Arguments>] [-wait] [-exportEnv]

The options are:

-jobId:        ID for the Job.

-executable:   Application to invoke on the Execute Node.

-args:         Command line arguments to the executable.

-wait:         Make the tool wait (block) until the job has finished executing.

-exportEnv:    Export the environment of the submit user. The environment
               variables and their values will be dumped into a file (using
               'env'). This file will be uploaded to the remote Execution
               Node and can be sourced by the application as follows, e.g.:

               source $_CONDOR_SCRATCH_DIR/env.bash

               Note, it is possible for a client to submit any environment
               by specifying the environment variables in a file, which will
               be scheduled for upload
    

EXECUTION ENVIRONMENT:
The tool prepares to run the DAG in the following way:

1. It creates a temporary directory, which will host the files, pertinent
to the job. This will be located in:

  $BQS_EXEC_ROOT/<User>/exec/<Job ID>

- where <Job ID> is the ID of the Job allocated by the user. The tool will
concatenate a timestamp to that ID to make it unique.

2. In this directory the following files will be contained:

  a. Job Submission File for the job (<Job ID>.cmd).
  b. Log files for the job (<Job ID>.err/.log/.out).

The temporary directory is visible and accessible from all nodes in a pool
configuration.

Note, no attempts are done to clean up the temporary directories automatically.
This can be done by invoking the tool vultur_clean_up. This could be invoked
periodically (system wide) via a cron job if desirable.
"""

import sys, os, time, commands
import vultur_utils

# Example of a job to be executed:
#
# $ /shared/system/i86/RH9.0/instroot/bin/exec_esorex -prefix mbiastst \
#   -recipe ocam_mbias -sof /shared/data/wfi/sof/BiasFramesWFI_EXT2.sof 


# Condor Job Submit Command.
CONDOR_JOB_SUBMIT = "condor_submit %s"


def correctUsage():
    """
    """
    print "\nCorrect usage is:\n\n" +\
          "$ vultur_submit -jobId <ID> -application <Application> " +\
          "[-args <Arguments>] [-noOfJobs <Number of Jobs>]\n\n\n" +\
          "MANUAL PAGE:" + __doc__ + "\n"


def submitJob(argv):
    """
    """
    user = os.environ["USER"]
    jobId = application = None
    args = ""
    noOfJobs = 1
    idx = 1
    while idx < len(sys.argv):
        par = sys.argv[idx].upper()
        if (par == "-JOBID"):
            idx += 1
            jobId = sys.argv[idx]
        elif (par == "-APPLICATION"):
            idx += 1
            application = sys.argv[idx]
        elif (par == "-ARGS"):
            idx += 1
            args = sys.argv[idx]
            args = args.replace("*Process", "$(Process)")
        elif (par == "-NOOFJOBS"):
            idx += 1
            noOfJobs = int(sys.argv[idx])
        elif ((par == "-H") or (par == "-HELP")):
            correctUsage()
            sys.exit(0)
        else:
            correctUsage()
            sys.exit(1)
        idx += 1
    
    if ((not jobId) or (not application)):
        correctUsage()
        sys.exit(1)

    # Generate Job Submission File.
    jobSubmisName, jobSubmis = vultur_utils.genJobSubmission(jobId, user,
                                                             application,
                                                             args, noOfJobs)

    # Submit the job.
    submitCmd = CONDOR_JOB_SUBMIT % jobSubmisName
    stat, out = commands.getstatusoutput(submitCmd)


if __name__ == '__main__':
    """
    Main function.
    """
    submitJob(sys.argv)
    sys.exit(0)

# EOF
